/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/* This is a very simple oggvorbis decoder which decodes stdin to stdout.
 */

#include "para.h"
#include <vorbis/vorbisfile.h>

#define BITS 16
#define ENDIAN 0
#define SIGN 1


/* derived from oggdec.c of the vorbis-tools-1.0.1 package */
int write_header(OggVorbis_File * vf)
{
#define WRITE_U32(buf, x) (buf)[0]= (unsigned char)((x)&0xff);\
	(buf)[1] = (unsigned char)(((x)>>8)&0xff);\
	(buf)[2] = (unsigned char)(((x)>>16)&0xff);\
	(buf)[3] = (unsigned char)(((x)>>24)&0xff);
#define WRITE_U16(buf, x) (buf)[0] = (unsigned char)((x)&0xff);

	static unsigned char headbuf[44];
	unsigned int size = 0x7fffffff;
	int channels = ov_info(vf, 0)->channels;
	int samplerate = ov_info(vf, 0)->rate;
	int bytespersec = channels * samplerate * BITS / 8;
	int align = channels * BITS / 8;

	memcpy(headbuf, "RIFF", 4);
	WRITE_U32(headbuf + 4, size - 8);
	memcpy(headbuf + 8, "WAVE", 4);
	memcpy(headbuf + 12, "fmt ", 4);
	WRITE_U32(headbuf + 16, 16);
	WRITE_U16(headbuf + 20, 1);	/* format */
	WRITE_U16(headbuf + 22, channels);
	WRITE_U32(headbuf + 24, samplerate);
	WRITE_U32(headbuf + 28, bytespersec);
	WRITE_U16(headbuf + 32, align);
	WRITE_U16(headbuf + 34, BITS);
	memcpy(headbuf + 36, "data", 4);
	WRITE_U32(headbuf + 40, size - 44);
	return write(STDOUT_FILENO, headbuf, 44) != 44 ? -1 : 1;
}

int main(int argc, char **argv)
{
	OggVorbis_File vf;
	char buf[8192];
	int ret;

	if (ov_open(stdin, &vf, NULL, 0) < 0)
		return -1;
	if (write_header(&vf) < 0)
		return -2;
	while ((ret = ov_read(&vf, buf, sizeof(buf), ENDIAN, BITS / 8, 
			SIGN, NULL))) {
		if (ret < 0) /* hole in data */
			continue;
		if (write(STDOUT_FILENO, buf, ret) != ret)
			return -3;
	}
	ov_clear(&vf);
	return ret;
}
