#include "para.h"
static int *signal_pipe;
static void (*log_fn)(int , char* ,...);

int para_signal_init(int *spipe, void (*lf)(int , char* ,...))
{
	log_fn = lf;
	signal_pipe = spipe;
	return pipe(spipe) < 0? -1 : 1;
}

/*
 * just write one integer to signal pipe
 */
static void generic_signal_handler(int s)
{
	write(signal_pipe[1], &s, sizeof(int));
	//fprintf(stderr, "got sig %i, write returned %d\n", s, ret);
}

pid_t para_reap_child(void)
{
	int status;
	pid_t pid = waitpid(-1, &status, WNOHANG);

	if (pid <= 0)
		return pid;
	if (WIFEXITED(status))
		log_fn(DEBUG, "%s: child %i exited. Exit status: %i\n",
			__func__, pid, WEXITSTATUS(status));
	else if (WIFSIGNALED(status))
		log_fn(DEBUG, "%s: child %i was killed by signal %i\n",
			__func__, pid, WTERMSIG(status));
	else 
		log_fn(WARNING, "%s: child %i terminated abormally\n",
			__func__, pid);
	return pid;
}

void para_reap_children(void)
{
	while (para_reap_child() > 0)
		; /* nothing */
}

int para_install_sighandler(int sig) 
{
	log_fn(DEBUG, "%s: catching signal %d\n", __func__, sig);
	return signal(sig, &generic_signal_handler) == SIG_ERR?  -1: 1;
}

/*
 * idea taken from libdaemon
 */
int para_next_signal()
{
	int s;
	ssize_t r;

	if ((r = read(signal_pipe[0], &s, sizeof(s)) == sizeof(s)) > 0) {
		log_fn(DEBUG, "%s: next signal: %d\n", __func__, s);
		return s;
	}
	return r < 0 && (errno != EAGAIN)? 0 : -1;
}
